package com.me.asteroids.systems;

import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;
import com.me.asteroids.Constants;
import com.me.asteroids.EntityFactory;
import com.me.asteroids.Utils;
import com.me.asteroids.components.AsteroidComponent;
import com.me.asteroids.components.model.Model;
import com.me.asteroids.components.ModelComponent;
import com.me.asteroids.components.PositionComponent;
import com.me.asteroids.components.VelocityComponent;
import com.me.common.ecs.BaseSystem;
import com.me.common.ecs.ComponentMapper;
import com.me.common.ecs.Engine;
import com.me.common.ecs.Entity;

import static com.me.asteroids.Constants.rand;

public class AsteroidSpawningSystem extends BaseSystem {

    private ComponentMapper<AsteroidComponent> asteroidMapper;
    private ComponentMapper<PositionComponent> positionMapper;
    private ComponentMapper<VelocityComponent> velocityMapper;
    private ComponentMapper<ModelComponent> modelMapper;

    private float asteroidSpawnDelay = 0f;

    public AsteroidSpawningSystem(Engine engine) {
        super(engine);
        asteroidMapper = engine.getComponentMapper(AsteroidComponent.class);
        positionMapper = engine.getComponentMapper(PositionComponent.class);
        velocityMapper = engine.getComponentMapper(VelocityComponent.class);
        modelMapper = engine.getComponentMapper(ModelComponent.class);
    }

    private Vector2 getRandomSpawnLocation(float asteroidWidth, float asteroidHeight) {
        float halfWidth = asteroidWidth / 2;
        float halfHeight = asteroidHeight / 2;

        Vector2[] spawnPerimeter = new Vector2[]{
                new Vector2(-halfWidth, -halfHeight),
                new Vector2(-halfWidth, Constants.HEIGHT + halfHeight),
                new Vector2(Constants.WIDTH + halfWidth, Constants.HEIGHT + halfHeight),
                new Vector2(Constants.WIDTH + halfWidth, -halfHeight),
        };

        float perimeterLength = (Constants.WIDTH + Constants.HEIGHT + asteroidWidth + asteroidHeight) * 2;
        float positionAlongPerimeter = rand.nextFloat(0, perimeterLength);

        return Utils.getPointAlongPolygon(spawnPerimeter, positionAlongPerimeter);
    }

    private void spawnAsteroid() {
        Entity asteroid = EntityFactory.createAsteroid(engine);

        Model model = modelMapper.get(asteroid).model;
        Rectangle aabb = model.getBoundingBox();

        Vector2 position
                = positionMapper.get(asteroid).position
                = getRandomSpawnLocation(aabb.getWidth(), aabb.getHeight());
        model.setPosition(position);

        VelocityComponent velocityComponent = velocityMapper.get(asteroid);
        velocityComponent.velocity = new Vector2().setToRandomDirection().scl(rand.nextFloat(75, 125));
        velocityComponent.angularVelocity = rand.nextFloat(-30, 30);

        asteroid.activate();
    }

    @Override
    public void process(float dt) {
        Array<Entity> entities = engine.getEntities();
        int asteroidCount = 0;
        for (Entity entity : entities) {
            // It's rather inefficient to have to check our entire entity list every frame
            // to count how many entities have a specific component. Maybe we should keep a count of
            // how many entites a given component?
            if (asteroidMapper.has(entity)) {
                asteroidCount++;
            }
        }

        if (asteroidCount++ < Constants.ASTEROID_SPAWN_COUNT && (asteroidSpawnDelay -= dt) <= 0) {
            spawnAsteroid();
            asteroidSpawnDelay = Constants.ASTEROID_SPAWN_DELAY;
        }
    }

}
